; ########################################
; NHWAVEGEN
; HAUPTQUELLTEXT
; (C) BY NILS HINTZE
; ########################################



; DebugLevel, 0 = Nur Fehler, 10 = Auch Infos, 20 = Auch periodische Infos
DebugLevel 10



; ########################################
;- Eingebundene Quelltexte
; ########################################

XIncludeFile "waveformgenerator.pbi"



; ########################################
;- Konstanten
; ########################################

;- Versionsinformationen
#AppName = "nhWaveGen"
#AppAuthor = "Nils Hintze"
#AppHomepage = "http://www.elektronikpage.net/nhwavegen/"
#AppVersionMajor = 0
#AppVersionMinor = 8
#AppVersionPatch = 0
#AppVersionBuild = #PB_Editor_BuildCount
#AppVersionDate = #PB_Compiler_Date

;- Anzahl von Einstellungsspeicherpltzen
#MemoryEntryCount = 10

;- Konfigurationsdateiname
#ConfigFileName = "nhwavegen.ini"

;- Hilfedateiname
#HelpFileName = "nhwavegen.htm"

;- Fenster
Enumeration
  ; Hauptfenster
  #Window_Main
  ; Ausgabegertefenster
  #Window_OutputDevice
  ; Infofenster
  #Window_About
EndEnumeration

;- Mens
Enumeration
  ; Menleiste Hauptfenster
  #Menu_MainWnd
  ; Popup-Men Amplitudeneinheit
  #Menu_AmpUnit
EndEnumeration

;- Meneintrge
Enumeration
  ; Menleiste Hauptfenster
  ; Mentitel "Funktionsgenerator"
  #MenuItem_MainWndStartStop
  #MenuItem_MainWndQuit
  ; Mentitel "Laden"
  #MenuItem_MainWndMemoryLoadFirst
  #MenuItem_MainWndMemoryLoadLast = #MenuItem_MainWndMemoryLoadFirst + #MemoryEntryCount - 1
  ; Mentitel "Speichern"
  #MenuItem_MainWndMemorySaveFirst
  #MenuItem_MainWndMemorySaveLast = #MenuItem_MainWndMemorySaveFirst + #MemoryEntryCount - 1
  ; Mentitel "?"
  #MenuItem_MainWndHelp
  #MenuItem_MainWndAbout
  ; Popup-Men Amplitudeneinheit
  #MenuItem_AmpUnitLinear
  #MenuItem_AmpUnitDecibel
EndEnumeration

;- Gadgets
Enumeration
  ; Gadgets Hauptfenster
  #ScrollArea_MainWndChannelGadgets
  ; Gadgets Ausgabegertefenster
  #Frame_OutDevWndOutputDevice
  #ComboBox_OutDevWndOutputDevice
  #Frame_OutDevWndSampleRate
  #ComboBox_OutDevWndSampleRate
  #Frame_OutDevWndResolution
  #ComboBox_OutDevWndResolution
  #Frame_OutDevWndBufferLength
  #ComboBox_OutDevWndBufferLength
  #Frame_OutDevWndChannelMask
  #ListIcon_OutDevWndChannelMask
  #Button_OutDevWndCancel
  #Button_OutDevWndOK
  ; Gadgets Infofenster
  #Text_AboutWndText
  #HyperLink_AboutWndLink
  #Button_AboutWndClose
EndEnumeration

;- Amplitudeneinheiten
Enumeration
  #AmplitudeUnit_Linear
  #AmplitudeUnit_Decibel
EndEnumeration


; ########################################
;- Strukturen
; ########################################

Structure Channel
  Index.i ; Index des Kanals in WaveformGenerator\Channel[]
  AmplitudeUnit.i ; Amplitudeneinheit
  AMMinAmpUnit.i ; Minimale AM-Amplitudeneinheit
  AMMaxAmpUnit.i ; Maximale AM-Amplitudeneinheit
  *Channel.WG_Channel ; Zeiger auf WaveformGenerator\Channel[\Index]
  *Settings.WG_ChannelSettings ; Zeiger auf WaveformGenerator\Channel[\Index]\Settings
  Update.q ; 
  Frame_Channel.i ; Gadgets...
  Text_WaveformLabel.i
  ComboBox_Waveform.i
  String_WaveformFile.i
  Button_WaveformFileSelect.i
  Text_MuteLabel.i
  CheckBox_Mute.i
  Text_PhaseShiftLabel.i
  TrackBar_PhaseShift.i
  String_PhaseShift.i
  Text_PhaseShiftUnit.i
  Text_FrequencyLabel.i
  TrackBar_Frequency.i
  Button_FreqUpTen.i
  Button_FreqUpOne.i
  Button_FreqUpTenth.i
  Button_FreqDownTenth.i
  Button_FreqDownOne.i
  Button_FreqDownTen.i
  String_Frequency.i
  Text_FrequencyUnit.i
  CheckBox_FMEnable.i
  ComboBox_FMWaveform.i
;   String_FMFile.i
;   Button_FMFileSelect.i
  Text_FMFrequencyLabel.i
  String_FMFrequency.i
  Text_FMFrequencyUnit.i
  Text_FMMinFreqLabel.i
  String_FMMinFreq.i
  Text_FMMinFreqUnit.i
  Text_FMMaxFreqLabel.i
  String_FMMaxFreq.i
  Text_FMMaxFreqUnit.i
  Text_AmplitudeLabel.i
  TrackBar_Amplitude.i
  String_Amplitude.i
  Button_AmplitudeUnit.i
  CheckBox_AMEnable.i
  ComboBox_AMWaveform.i
  String_AMFile.i
  Button_AMFileSelect.i
  Text_AMFrequencyLabel.i
  String_AMFrequency.i
  Text_AMFrequencyUnit.i
  Text_AMMinAmpLabel.i
  String_AMMinAmp.i
  Button_AMMinAmpUnit.i
  Text_AMMaxAmpLabel.i
  String_AMMaxAmp.i
  Button_AMMaxAmpUnit.i
EndStructure

Structure AmpUnitMenuInfo
  *Unit.Integer
  *Channel.Channel
  Update.q
EndStructure

Structure Memory
  Name.s
  Settings.WG_ChannelSettings[#WaveformGenerator_MaximumChannelCount]
EndStructure



; ########################################
;- Variablen
; ########################################

Define WaveformGenerator.WaveformGenerator, AmpUnitMenuInfo.AmpUnitMenuInfo

NewList Channel.Channel()

Dim Memory.Memory(#MemoryEntryCount-1)



; ########################################
;- Prozeduren
; ########################################

; Versionsstring erzeugen
Macro VersionString(Major, Minor, Patch, Build)
  Str(Major)+"."+Str(Minor)+"."+Str(Patch)+"-"+Str(Build)
EndMacro

; Allgemeine Meldungen
Macro ErrorMessage(Message, Flags = #PB_MessageRequester_Ok)
  MessageRequester("Fehler", Message, (Flags)|#MB_ICONERROR)
EndMacro
Macro InfoMessage(Message, Flags = #PB_MessageRequester_Ok)
  MessageRequester("Information", Message, (Flags)|#MB_ICONINFORMATION)
EndMacro
Macro WarningMessage(Message, Flags = #PB_MessageRequester_Ok)
  MessageRequester("Warnung", Message, (Flags)|#MB_ICONWARNING)
EndMacro
Macro QuestionMessage(Message, Flags = #PB_MessageRequester_Ok)
  MessageRequester("Frage", Message, (Flags)|#MB_ICONQUESTION)
EndMacro

; Popup-Men unterhalb eines Gadgets anzeigen
Procedure DisplayPopupMenuBelowGadget(Menu, WindowID, Gadget)
  Protected Position.POINT
  MapWindowPoints_(GadgetID(Gadget), #Null, @Position, 1)
  DisplayPopupMenu(Menu, WindowID, Position\x, Position\y+GadgetHeight(Gadget))
EndProcedure

; bernommen aus NhFormatString.pbi
Procedure FS_GetExp(Double.d, Base.d = 10)
  If Double = 0 Or 1.0 * IsNAN(Double) Or 1.0 * IsInfinity(Double) ; Multiplikation mit 1.0 ist erforderlich, damit die Funktionen mit doppelter Genauigkeit arbeiten
    ProcedureReturn 0
  EndIf
  ProcedureReturn 1.0 * Round(Log(Abs(Double))/Log(Base), #PB_Round_Down)
EndProcedure
Procedure.s FS_StrD(Double.d, Digits = 10, Trim = #True)
  Protected String$, *c.Character, l, l2, p
  Digits - (FS_GetExp(Double) + 1)
  If Digits < 0
    Digits = 0
  EndIf
  String$ = StrD(Double, Digits)
  If Not Trim
    ProcedureReturn String$
  EndIf
  *c = @String$
  While *c\c
    l + 1
    If *c\c = '.'
      p = #True
    EndIf
    If (*c\c <> '0' And *c\c <> '.') Or Not p
      l2 = l
    EndIf
    *c + SizeOf(Character)
  Wend
  ProcedureReturn Left(String$, l2)
EndProcedure

; Wie ValD, akzeptiert jedoch Komma als Dezimaltrennzeichen
Procedure.d ValDComma(String$)
  *c.Character = @String$
  While *c\c
    If *c\c = ','
      *c\c = '.'
      Break
    EndIf
    *c + SizeOf(Character)
  Wend
  ProcedureReturn ValD(String$)
EndProcedure

; Gadget-Eintrag anhand der Daten auswhlen
Procedure SetGadgetStateByItemData(Gadget, ItemData, DefaultState = -1)
  For Item = CountGadgetItems(Gadget) - 1 To 0 Step -1
    If GetGadgetItemData(Gadget, Item) = ItemData
      SetGadgetState(Gadget, Item)
      ProcedureReturn Item
    EndIf
  Next
  SetGadgetState(Gadget, DefaultState)
  ProcedureReturn DefaultState
EndProcedure

; Index des ersten gesetzten Bits in Integer ermitteln und dabei SkipBits Bits ignorieren
Procedure NextSetBitIndexInteger(Integer, SkipBits = 0)
  CompilerIf #PB_Compiler_Processor <> #PB_Processor_x86
    CompilerError "Assemblercode fr vorzeichenlosen Rechts-Shift ist nicht fr diese Prozessorarchitektur vorgesehen!"
  CompilerEndIf
  For n = 0 To 31
    If (Integer & 1)
      If SkipBits = 0
        ProcedureReturn n
      EndIf
      SkipBits - 1
    EndIf
    EnableASM
      SHR Integer, 1
    DisableASM
  Next
  ProcedureReturn 0
EndProcedure

; String aus einem Array von Strings aus dem Speicher lesen
Procedure.s PeekSArray(*Memory.Character, Index)
  While Index > 0
    If *Memory\c = 0
      Index - 1
    EndIf
    *Memory + SizeOf(Character)
  Wend
  ProcedureReturn PeekS(*Memory)
EndProcedure

; Wert von Var auf Bereich Min...Max begrenzen
; Min und Max sollten Konstanten sein!
Macro Limit(Var, Min, Max)
  If Var < Min
    Var = Min
  ElseIf Var > Max
    Var = Max
  EndIf
EndMacro

; WG_ChannelSettings-Struktur aus Preference lesen
Procedure ReadPreferenceChannelSettings(*CS.WG_ChannelSettings)
  With *CS
    \Flags       = ReadPreferenceInteger("Flags",       #WG_Mute)
    \Waveform    = ReadPreferenceInteger("Waveform",    #WG_Sine)
    \PhaseShift  = ReadPreferenceDouble ("PhaseShift",  0)
    \Frequency   = ReadPreferenceDouble ("Frequency",   440)
    \FMWaveform  = ReadPreferenceInteger("FMWaveform",  #WG_Sine)
    \FMFrequency = ReadPreferenceDouble ("FMFrequency", 1)
    \FMMinFreq   = ReadPreferenceDouble ("FMMinFreq",   440)
    \FMMaxFreq   = ReadPreferenceDouble ("FMMaxFreq",   880)
    \Amplitude   = ReadPreferenceDouble ("Amplitude",   0.1)
    \AMWaveform  = ReadPreferenceInteger("AMWaveform",  #WG_Sine)
    \AMFrequency = ReadPreferenceDouble ("AMFrequency", 1)
    \AMMinAmp    = ReadPreferenceDouble ("AMMinAmp",    0.001)
    \AMMaxAmp    = ReadPreferenceDouble ("AMMaxAmp",    0.1)
  EndWith
EndProcedure

; WG_ChannelSettings-Struktur in Preference schreiben
Procedure WritePreferenceChannelSettings(*CS.WG_ChannelSettings)
  With *CS
    WritePreferenceInteger("Flags",       \Flags)
    WritePreferenceInteger("Waveform",    \Waveform)
    WritePreferenceDouble ("PhaseShift",  \PhaseShift)
    WritePreferenceDouble ("Frequency",   \Frequency)
    WritePreferenceInteger("FMWaveform",  \FMWaveform)
    WritePreferenceDouble ("FMFrequency", \FMFrequency)
    WritePreferenceDouble ("FMMinFreq",   \FMMinFreq)
    WritePreferenceDouble ("FMMaxFreq",   \FMMaxFreq)
    WritePreferenceDouble ("Amplitude",   \Amplitude)
    WritePreferenceInteger("AMWaveform",  \AMWaveform)
    WritePreferenceDouble ("AMFrequency", \AMFrequency)
    WritePreferenceDouble ("AMMinAmp",    \AMMinAmp)
    WritePreferenceDouble ("AMMaxAmp",    \AMMaxAmp)
  EndWith
EndProcedure

; Konfigurationsdatei lesen
Procedure LoadConfig(FileName$)
  
  Shared WaveformGenerator, Memory()
  
  OpenPreferences(FileName$)
  
  ; Ausgabegert
  PreferenceGroup("OutputDevice")
  WaveformGenerator\DeviceID      = ReadPreferenceInteger("OutputDevice", #WAVE_MAPPER)
  WaveformGenerator\SampleRate    = ReadPreferenceInteger("SampleRate",   44100)
  WaveformGenerator\BitsPerSample = ReadPreferenceInteger("Resolution",   16)
  WaveformGenerator\BufferLength  = ReadPreferenceInteger("BufferLength", 250)
  WaveformGenerator\ChannelMask   = ReadPreferenceInteger("ChannelMask",  #SPEAKER_FRONT_LEFT|#SPEAKER_FRONT_RIGHT)
  
  ; Kanaleinstellungen
  For n = 0 To #MemoryEntryCount - 1
    PreferenceGroup("Memory"+Str(n))
    Memory(n)\Name = ReadPreferenceString("Name", "Unbenannt")
    For i = 0 To #WaveformGenerator_MaximumChannelCount - 1
      PreferenceGroup("Memory"+Str(n)+"Channel"+Str(i))
      ReadPreferenceChannelSettings(@Memory(n)\Settings[i])
    Next
  Next
  
  ClosePreferences()

EndProcedure

; Konfigurationsdatei schreiben
Procedure SaveConfig(FileName$)
  
  Shared WaveformGenerator, Memory()
  
  If CreatePreferences(FileName$, #PB_Preference_GroupSeparator)
    
    ; Ausgabegert
    PreferenceGroup("OutputDevice")
    WritePreferenceInteger("OutputDevice", WaveformGenerator\DeviceID)
    WritePreferenceInteger("SampleRate",   WaveformGenerator\SampleRate)
    WritePreferenceInteger("Resolution",   WaveformGenerator\BitsPerSample)
    WritePreferenceInteger("BufferLength", WaveformGenerator\BufferLength)
    WritePreferenceInteger("ChannelMask",  WaveformGenerator\ChannelMask)
    
    ; Kanaleinstellungen
    For n = 0 To #MemoryEntryCount - 1
      PreferenceGroup("Memory"+Str(n))
      WritePreferenceString("Name", Memory(n)\Name)
      For i = 0 To #WaveformGenerator_MaximumChannelCount - 1
        PreferenceGroup("Memory"+Str(n)+"Channel"+Str(i))
        WritePreferenceChannelSettings(@Memory(n)\Settings[i])
      Next
    Next
  
    ClosePreferences()
    
    ProcedureReturn #True
    
  EndIf
  
  ProcedureReturn #False

EndProcedure

; Infofenster
Procedure AboutWindow()
  If OpenWindow(#Window_About, 0, 0, 320, 160, "ber "+#AppName, #PB_Window_TitleBar|#PB_Window_WindowCentered, WindowID(#Window_Main))
    TextGadget     (#Text_AboutWndText,        0,  10, 330,  90, #AppName+#LF$+#LF$+"Soundkarten-Funktionsgenerator"+#LF$+"von "+#AppAuthor+#LF$+#LF$+
                                                                 "Version "+VersionString(#AppVersionMajor, #AppVersionMinor, #AppVersionPatch, #AppVersionBuild)+
                                                                 " ("+FormatDate("%dd.%mm.%yyyy", #AppVersionDate)+")", #PB_Text_Center)
    HyperLinkGadget(#HyperLink_AboutWndLink,  10, 100, 300,  20, #AppHomepage, $0000FF, #PB_HyperLink_Underline)
    SetGadgetColor(#HyperLink_AboutWndLink, #PB_Gadget_FrontColor, $FF0000)
    ButtonGadget   (#Button_AboutWndClose,   110, 130, 100,  20, "OK")
  EndIf
EndProcedure

; Ausgabegertefenster
Procedure OutputDeviceWindow()
  
  Shared WaveformGenerator
  
  If OpenWindow(#Window_OutputDevice, 0, 0, 415, 250, "Ausgabegert konfigurieren", #PB_Window_TitleBar|#PB_Window_WindowCentered, WindowID(#Window_Main))
    
    ; Gadgets
    FrameGadget   (#Frame_OutDevWndOutputDevice,      5,   5, 200,  50, "Ausgabegert")
    ComboBoxGadget(#ComboBox_OutDevWndOutputDevice,  15,  25, 180,  20)
    FrameGadget   (#Frame_OutDevWndSampleRate,        5,  60, 200,  50, "Abtastrate")
    ComboBoxGadget(#ComboBox_OutDevWndSampleRate,    15,  80, 180,  20)
    FrameGadget   (#Frame_OutDevWndResolution,        5, 115, 200,  50, "Auflsung")
    ComboBoxGadget(#ComboBox_OutDevWndResolution,    15, 135, 180,  20)
    FrameGadget   (#Frame_OutDevWndBufferLength,      5, 170, 200,  50, "Pufferlnge")
    ComboBoxGadget(#ComboBox_OutDevWndBufferLength,  15, 190, 180,  20)
    FrameGadget   (#Frame_OutDevWndChannelMask,     210,   5, 200, 215, "Kanle")
    ListIconGadget(#ListIcon_OutDevWndChannelMask,  220,  25, 180, 185, "Kanal", 140, #PB_ListIcon_CheckBoxes)
    ButtonGadget  (#Button_OutDevWndCancel,         105, 225, 100,  20, "Abbrechen")
    ButtonGadget  (#Button_OutDevWndOK,             210, 225, 100,  20, "OK")
    
    ; Ausgabegert
    Item = 0
    NumDevs = waveOutGetNumDevs_()
    For n = #WAVE_MAPPER To NumDevs - 1
      If waveOutGetDevCaps_(n, @WOC.WAVEOUTCAPS, SizeOf(WAVEOUTCAPS)) = #MMSYSERR_NOERROR
        AddGadgetItem(#ComboBox_OutDevWndOutputDevice, Item, PeekS(@WOC\szPname))
        SetGadgetItemData(#ComboBox_OutDevWndOutputDevice, Item, n)
        Item + 1
      EndIf
    Next
    SetGadgetStateByItemData(#ComboBox_OutDevWndOutputDevice, WaveformGenerator\DeviceID, 0)
    
    ; Abtastrate
    AddGadgetItem    (#ComboBox_OutDevWndSampleRate, 0, "8000 Hz")
    SetGadgetItemData(#ComboBox_OutDevWndSampleRate, 0, 8000)
    AddGadgetItem    (#ComboBox_OutDevWndSampleRate, 1, "11025 Hz")
    SetGadgetItemData(#ComboBox_OutDevWndSampleRate, 1, 11025)
    AddGadgetItem    (#ComboBox_OutDevWndSampleRate, 2, "22050 Hz")
    SetGadgetItemData(#ComboBox_OutDevWndSampleRate, 2, 22050)
    AddGadgetItem    (#ComboBox_OutDevWndSampleRate, 3, "44100 Hz")
    SetGadgetItemData(#ComboBox_OutDevWndSampleRate, 3, 44100)
    AddGadgetItem    (#ComboBox_OutDevWndSampleRate, 4, "48000 Hz")
    SetGadgetItemData(#ComboBox_OutDevWndSampleRate, 4, 48000)
    AddGadgetItem    (#ComboBox_OutDevWndSampleRate, 5, "96000 Hz")
    SetGadgetItemData(#ComboBox_OutDevWndSampleRate, 5, 96000)
    AddGadgetItem    (#ComboBox_OutDevWndSampleRate, 6, "192000 Hz")
    SetGadgetItemData(#ComboBox_OutDevWndSampleRate, 6, 192000)
    SetGadgetStateByItemData(#ComboBox_OutDevWndSampleRate, WaveformGenerator\SampleRate, 3)
    
    ; Auflsung
    For n = 0 To 3
      AddGadgetItem(#ComboBox_OutDevWndResolution, n, Str((n+1)*8)+" Bit")
      SetGadgetItemData(#ComboBox_OutDevWndResolution, n, (n+1)*8)
    Next
    SetGadgetStateByItemData(#ComboBox_OutDevWndResolution, WaveformGenerator\BitsPerSample, 1)
    
    ; Pufferlnge
    AddGadgetItem    (#ComboBox_OutDevWndBufferLength, 0, "50 ms")
    SetGadgetItemData(#ComboBox_OutDevWndBufferLength, 0, 50)
    AddGadgetItem    (#ComboBox_OutDevWndBufferLength, 1, "100 ms")
    SetGadgetItemData(#ComboBox_OutDevWndBufferLength, 1, 100)
    AddGadgetItem    (#ComboBox_OutDevWndBufferLength, 2, "250 ms")
    SetGadgetItemData(#ComboBox_OutDevWndBufferLength, 2, 250)
    AddGadgetItem    (#ComboBox_OutDevWndBufferLength, 3, "500 ms")
    SetGadgetItemData(#ComboBox_OutDevWndBufferLength, 3, 500)
    AddGadgetItem    (#ComboBox_OutDevWndBufferLength, 4, "1000 ms")
    SetGadgetItemData(#ComboBox_OutDevWndBufferLength, 4, 1000)
    SetGadgetStateByItemData(#ComboBox_OutDevWndBufferLength, WaveformGenerator\BufferLength, 2)
    
    ; Kanalmaske
    Restore ChannelNames
    For n = 0 To #WaveformGenerator_ChannelMaskBitSize - 1
      Read.s Name$
      AddGadgetItem(#ListIcon_OutDevWndChannelMask, n, Name$)
      If WaveformGenerator\ChannelMask & (1 << n)
        SetGadgetItemState(#ListIcon_OutDevWndChannelMask, n, #PB_ListIcon_Checked)
      EndIf
    Next

  EndIf
  
EndProcedure

; Einstellungen im Ausgabegertefenster einlesen
Procedure GetOutputDevice()
  
  Shared WaveformGenerator
  
  ; Ausgabegert
  Item = GetGadgetState(#ComboBox_OutDevWndOutputDevice)
  If Item = -1
    ProcedureReturn #False
  EndIf
  WaveformGenerator\DeviceID = GetGadgetItemData(#ComboBox_OutDevWndOutputDevice, Item)
  
  ; Abtastrate
  Item = GetGadgetState(#ComboBox_OutDevWndSampleRate)
  If Item = -1
    ProcedureReturn #False
  EndIf
  WaveformGenerator\SampleRate = GetGadgetItemData(#ComboBox_OutDevWndSampleRate, Item)
  
  ; Auflsung
  Item = GetGadgetState(#ComboBox_OutDevWndResolution)
  If Item = -1
    ProcedureReturn #False
  EndIf
  WaveformGenerator\BitsPerSample = GetGadgetItemData(#ComboBox_OutDevWndResolution, Item)
  
  ; Pufferlnge
  Item = GetGadgetState(#ComboBox_OutDevWndBufferLength)
  If Item = -1
    ProcedureReturn #False
  EndIf
  WaveformGenerator\BufferLength = GetGadgetItemData(#ComboBox_OutDevWndBufferLength, Item)
  
  ; Kanalmaske
  WaveformGenerator\ChannelMask = 0
  WaveformGenerator\ChannelCount = 0
  For n = 0 To #WaveformGenerator_ChannelMaskBitSize - 1
    If GetGadgetItemState(#ListIcon_OutDevWndChannelMask, n) & #PB_ListIcon_Checked
      WaveformGenerator\ChannelMask | (1 << n)
      WaveformGenerator\ChannelCount + 1
    EndIf
  Next  
  If Not (WaveformGenerator\ChannelCount >= 1 And WaveformGenerator\ChannelCount <= #WaveformGenerator_MaximumChannelCount)
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn #True
  
EndProcedure

; Hauptfenster aktualisieren
#UMW_Size = $01
#UMW_WaveformGeneratorEnable = $02
#UMW_Memory = $04
Procedure UpdateMainWindow(Update)
  
  Shared WaveformGenerator, Memory()
  
  ; 
  If Update & #UMW_WaveformGeneratorEnable
    If WaveformGeneratorEnabled(@WaveformGenerator)
      SetMenuItemText(#Menu_MainWnd, #MenuItem_MainWndStartStop, "&Anhalten"+#TAB$+"Strg+R")
    Else
      SetMenuItemText(#Menu_MainWnd, #MenuItem_MainWndStartStop, "&Starten..."+#TAB$+"Strg+R")
    EndIf
  EndIf
  
  ; 
  If Update & #UMW_WaveformGeneratorEnable
    If WaveformGeneratorEnabled(@WaveformGenerator)
      State = 0
    Else
      State = 1
    EndIf
    For n = 0 To #MemoryEntryCount - 1
      DisableMenuItem(#Window_Main, #MenuItem_MainWndMemoryLoadFirst+n, State)
      DisableMenuItem(#Window_Main, #MenuItem_MainWndMemorySaveFirst+n, State)
    Next
  EndIf
  
  ; Namen der gespeicherten Einstellungen
  If Update & #UMW_Memory
    For n = 0 To #MemoryEntryCount - 1
      If n < 10
        Key = (n + 1) % 10
        SetMenuItemText(#Window_Main, #MenuItem_MainWndMemoryLoadFirst+n, Memory(n)\Name+#TAB$+"Strg+&"+Str(Key))
        SetMenuItemText(#Window_Main, #MenuItem_MainWndMemorySaveFirst+n, Memory(n)\Name+#TAB$+"Alt+&"+Str(Key))
      Else
        SetMenuItemText(#Window_Main, #MenuItem_MainWndMemoryLoadFirst+n, Memory(n)\Name)
        SetMenuItemText(#Window_Main, #MenuItem_MainWndMemorySaveFirst+n, Memory(n)\Name)
      EndIf
    Next
  EndIf
  
  ; Auenabmessungen des ScrollAreaGadgets
  If Update & #UMW_Size
    ResizeGadget(#ScrollArea_MainWndChannelGadgets, 0, 0, WindowWidth(#Window_Main), WindowHeight(#Window_Main)-GetSystemMetrics_(#SM_CYMENU))
  EndIf
  
  ; Innenabmessungen des ScrollAreaGadgets
  If Update & #UMW_WaveformGeneratorEnable
    If WaveformGeneratorEnabled(@WaveformGenerator)
      SetGadgetAttribute(#ScrollArea_MainWndChannelGadgets, #PB_ScrollArea_InnerWidth, 850)
      SetGadgetAttribute(#ScrollArea_MainWndChannelGadgets, #PB_ScrollArea_InnerHeight, 5+WaveformGenerator\ChannelCount*205)
    Else
      SetGadgetAttribute(#ScrollArea_MainWndChannelGadgets, #PB_ScrollArea_InnerWidth, 50)
      SetGadgetAttribute(#ScrollArea_MainWndChannelGadgets, #PB_ScrollArea_InnerHeight, 50)
    EndIf
  EndIf

EndProcedure

; ComboBox mit Wellenformen fllen
Macro InitWaveformComboBox(Gadget, Arbitrary = #True)
  AddGadgetItem    (Gadget, 0, "Sinus")
  SetGadgetItemData(Gadget, 0, #WG_Sine)
  AddGadgetItem    (Gadget, 1, "Dreieck")
  SetGadgetItemData(Gadget, 1, #WG_Triangle)
  AddGadgetItem    (Gadget, 2, "Sgezahn fallend")
  SetGadgetItemData(Gadget, 2, #WG_SawtoothFalling)
  AddGadgetItem    (Gadget, 3, "Sgezahn steigend")
  SetGadgetItemData(Gadget, 3, #WG_SawtoothRising)
  AddGadgetItem    (Gadget, 4, "Rechteck")
  SetGadgetItemData(Gadget, 4, #WG_Rectangle)
  CompilerIf Arbitrary
    AddGadgetItem    (Gadget, 5, "Datei")
    SetGadgetItemData(Gadget, 5, #WG_Arbitrary)
  CompilerEndIf
EndMacro

; Kanal-Gadgets erstellen
Procedure CreateChannelGadgets(*C.Channel)
  
  Shared WaveformGenerator

  With *C
    
    xPos = 5
    yPos = 5 + \Index * 205
    
    x = xPos + 0 : y = yPos + 0
    \Frame_Channel = FrameGadget(#PB_Any, x+  0, y+  0, 840, 200, "Kanal "+Str(\Index+1)+" ("+PeekSArray(?ChannelNames, NextSetBitIndexInteger(WaveformGenerator\ChannelMask, \Index))+")")
    
    x = xPos + 10 : y = yPos + 20
    \Text_WaveformLabel        = TextGadget    (#PB_Any, x+  0, y+  0, 200,  20, "Wellenform", #PB_Text_Center|#PB_Text_Border)
    \ComboBox_Waveform         = ComboBoxGadget(#PB_Any, x+  0, y+ 25, 200,  20)
    \String_WaveformFile       = StringGadget  (#PB_Any, x+  0, y+ 50, 165,  20, "", #PB_String_ReadOnly)
    \Button_WaveformFileSelect = ButtonGadget  (#PB_Any, x+170, y+ 50,  30,  20, "...")
    
    x = xPos + 10 : y = yPos + 95
    \Text_MuteLabel = TextGadget    (#PB_Any, x+  0, y+  0, 200,  20, "Stummschaltung", #PB_Text_Center|#PB_Text_Border)
    \CheckBox_Mute  = CheckBoxGadget(#PB_Any, x+  0, y+ 25, 200,  20, "Stummschaltung", #PB_CheckBox_Center)
    
    x = xPos + 10 : y = yPos + 145
    \Text_PhaseShiftLabel = TextGadget    (#PB_Any, x+  0, y+  0, 200,  20, "Phasenverschiebung", #PB_Text_Center|#PB_Text_Border)
    \TrackBar_PhaseShift  = TrackBarGadget(#PB_Any, x+  0, y+ 25,  95,  20, 0, 180)
    \String_PhaseShift    = StringGadget  (#PB_Any, x+100, y+ 25,  65,  20, "")
    \Text_PhaseShiftUnit  = TextGadget    (#PB_Any, x+170, y+ 25,  30,  20, "")
    
    x = xPos + 215 : y = yPos + 20
    \Text_FrequencyLabel  = TextGadget    (#PB_Any, x+  0, y+  0, 305,  20, "Frequenz", #PB_Text_Center|#PB_Text_Border)
    \TrackBar_Frequency   = TrackBarGadget(#PB_Any, x+ 20, y+ 25,  30, 120, 0, 300, #PB_TrackBar_Vertical)
    \Button_FreqUpTen     = ButtonGadget  (#PB_Any, x+ 70, y+ 25,  30,  20, "+10")
    \Button_FreqUpOne     = ButtonGadget  (#PB_Any, x+ 70, y+ 45,  30,  20, "+1")
    \Button_FreqUpTenth   = ButtonGadget  (#PB_Any, x+ 70, y+ 65,  30,  20, "+0,1")
    \Button_FreqDownTenth = ButtonGadget  (#PB_Any, x+ 70, y+ 85,  30,  20, "-0,1")
    \Button_FreqDownOne   = ButtonGadget  (#PB_Any, x+ 70, y+105,  30,  20, "-1")
    \Button_FreqDownTen   = ButtonGadget  (#PB_Any, x+ 70, y+125,  30,  20, "-10")
    \String_Frequency     = StringGadget  (#PB_Any, x+  0, y+150,  65,  20, "")
    \Text_FrequencyUnit   = TextGadget    (#PB_Any, x+ 70, y+150,  30,  20, "Hz")
    
    x = xPos + 320 : y = yPos + 45
    \CheckBox_FMEnable     = CheckBoxGadget(#PB_Any, x+  0, y+  0, 200,  20, "Frequenzmodulation", #PB_CheckBox_Center)
    \ComboBox_FMWaveform   = ComboBoxGadget(#PB_Any, x+  0, y+ 25, 200,  20)
;     \String_FMFile         = StringGadget  (#PB_Any, x+  0, y+ 50, 165,  20, "", #PB_String_ReadOnly)
;     \Button_FMFileSelect   = ButtonGadget  (#PB_Any, x+170, y+ 50,  30,  20, "...")
    \Text_FMFrequencyLabel = TextGadget    (#PB_Any, x+  0, y+ 75,  85,  20, "Frequenz:")
    \String_FMFrequency    = StringGadget  (#PB_Any, x+ 90, y+ 75,  75,  20, "")
    \Text_FMFrequencyUnit  = TextGadget    (#PB_Any, x+170, y+ 75,  30,  20, "Hz")
    \Text_FMMinFreqLabel   = TextGadget    (#PB_Any, x+  0, y+100,  85,  20, "Minimum:")
    \String_FMMinFreq      = StringGadget  (#PB_Any, x+ 90, y+100,  75,  20, "")
    \Text_FMMinFreqUnit    = TextGadget    (#PB_Any, x+170, y+100,  30,  20, "Hz")
    \Text_FMMaxFreqLabel   = TextGadget    (#PB_Any, x+  0, y+125,  85,  20, "Maximum:")
    \String_FMMaxFreq      = StringGadget  (#PB_Any, x+ 90, y+125,  75,  20, "")
    \Text_FMMaxFreqUnit    = TextGadget    (#PB_Any, x+170, y+125,  30,  20, "Hz")
    
    x = xPos + 525 : y = yPos + 20
    \Text_AmplitudeLabel  = TextGadget    (#PB_Any, x+  0, y+  0, 305,  20, "Amplitude", #PB_Text_Center|#PB_Text_Border)
    \TrackBar_Amplitude   = TrackBarGadget(#PB_Any, x+ 35, y+ 25,  30, 120, 0, 300, #PB_TrackBar_Vertical)
    \String_Amplitude     = StringGadget  (#PB_Any, x+  0, y+150,  65,  20, "")
    \Button_AmplitudeUnit = ButtonGadget  (#PB_Any, x+ 70, y+150,  30,  20, "")
    
    x = xPos + 630 : y = yPos + 45
    \CheckBox_AMEnable     = CheckBoxGadget(#PB_Any, x+  0, y+  0, 200,  20, "Amplitudenmodulation", #PB_CheckBox_Center)
    \ComboBox_AMWaveform   = ComboBoxGadget(#PB_Any, x+  0, y+ 25, 200,  20)
    \String_AMFile         = StringGadget  (#PB_Any, x+  0, y+ 50, 165,  20, "", #PB_String_ReadOnly)
    \Button_AMFileSelect   = ButtonGadget  (#PB_Any, x+170, y+ 50,  30,  20, "...")
    \Text_AMFrequencyLabel = TextGadget    (#PB_Any, x+  0, y+ 75,  85,  20, "Frequenz:")
    \String_AMFrequency    = StringGadget  (#PB_Any, x+ 90, y+ 75,  75,  20, "")
    \Text_AMFrequencyUnit  = TextGadget    (#PB_Any, x+170, y+ 75,  30,  20, "Hz")
    \Text_AMMinAmpLabel    = TextGadget    (#PB_Any, x+  0, y+100,  85,  20, "Minimum:")
    \String_AMMinAmp       = StringGadget  (#PB_Any, x+ 90, y+100,  75,  20, "")
    \Button_AMMinAmpUnit   = ButtonGadget  (#PB_Any, x+170, y+100,  30,  20, "")
    \Text_AMMaxAmpLabel    = TextGadget    (#PB_Any, x+  0, y+125,  85,  20, "Maximum:")
    \String_AMMaxAmp       = StringGadget  (#PB_Any, x+ 90, y+125,  75,  20, "")
    \Button_AMMaxAmpUnit   = ButtonGadget  (#PB_Any, x+170, y+125,  30,  20, "")
    
    InitWaveformComboBox(\ComboBox_Waveform)
    InitWaveformComboBox(\ComboBox_FMWaveform, #False)
    InitWaveformComboBox(\ComboBox_AMWaveform)
    
  EndWith
  
EndProcedure

; Kanal-Gadgets freigeben
Procedure FreeChannelGadgets(*C.Channel)
  
  With *C
    
    FreeGadget(\Frame_Channel)
    
    FreeGadget(\Text_WaveformLabel)
    FreeGadget(\ComboBox_Waveform)
    FreeGadget(\String_WaveformFile)
    FreeGadget(\Button_WaveformFileSelect)
    
    FreeGadget(\Text_MuteLabel)
    FreeGadget(\CheckBox_Mute)
    
    FreeGadget(\Text_PhaseShiftLabel)
    FreeGadget(\TrackBar_PhaseShift)
    FreeGadget(\String_PhaseShift)
    FreeGadget(\Text_PhaseShiftUnit)
    
    FreeGadget(\Text_FrequencyLabel)
    FreeGadget(\TrackBar_Frequency)
    FreeGadget(\Button_FreqUpTen)
    FreeGadget(\Button_FreqUpOne)
    FreeGadget(\Button_FreqUpTenth)
    FreeGadget(\Button_FreqDownTenth)
    FreeGadget(\Button_FreqDownOne)
    FreeGadget(\Button_FreqDownTen)
    FreeGadget(\String_Frequency)
    FreeGadget(\Text_FrequencyUnit)
    
    FreeGadget(\CheckBox_FMEnable)
    FreeGadget(\ComboBox_FMWaveform)
;     FreeGadget(\String_FMFile)
;     FreeGadget(\Button_FMFileSelect)
    FreeGadget(\Text_FMFrequencyLabel)
    FreeGadget(\String_FMFrequency)
    FreeGadget(\Text_FMFrequencyUnit)
    FreeGadget(\Text_FMMinFreqLabel)
    FreeGadget(\String_FMMinFreq)
    FreeGadget(\Text_FMMinFreqUnit)
    FreeGadget(\Text_FMMaxFreqLabel)
    FreeGadget(\String_FMMaxFreq)
    FreeGadget(\Text_FMMaxFreqUnit)
    
    FreeGadget(\Text_AmplitudeLabel)
    FreeGadget(\TrackBar_Amplitude)
    FreeGadget(\String_Amplitude)
    FreeGadget(\Button_AmplitudeUnit)
    
    FreeGadget(\CheckBox_AMEnable)
    FreeGadget(\ComboBox_AMWaveform)
    FreeGadget(\String_AMFile)
    FreeGadget(\Button_AMFileSelect)
    FreeGadget(\Text_AMFrequencyLabel)
    FreeGadget(\String_AMFrequency)
    FreeGadget(\Text_AMFrequencyUnit)
    FreeGadget(\Text_AMMinAmpLabel)
    FreeGadget(\String_AMMinAmp)
    FreeGadget(\Button_AMMinAmpUnit)
    FreeGadget(\Text_AMMaxAmpLabel)
    FreeGadget(\String_AMMaxAmp)
    FreeGadget(\Button_AMMaxAmpUnit)
    
  EndWith
  
EndProcedure

; Kanal-Gadgets aktualisieren
#UCG_Flags               = $000000001
#UCG_Waveform            = $000000002
#UCG_ComboBox_Waveform   = $000000004
#UCG_CheckBox_Mute       = $000000008
#UCG_PhaseShift          = $000000010
#UCG_TrackBar_PhaseShift = $000000020
#UCG_String_PhaseShift   = $000000040
#UCG_Frequency           = $000000080
#UCG_TrackBar_Frequency  = $000000100
#UCG_String_Frequency    = $000000200
#UCG_CheckBox_FMEnable   = $000000400
#UCG_FMWaveform          = $000000800
#UCG_ComboBox_FMWaveform = $000001000
#UCG_FMFrequency         = $000002000
#UCG_String_FMFrequency  = $000004000
#UCG_FMMinFreq           = $000008000
#UCG_String_FMMinFreq    = $000010000
#UCG_FMMaxFreq           = $000020000
#UCG_String_FMMaxFreq    = $000040000
#UCG_Amplitude           = $000080000
#UCG_TrackBar_Amplitude  = $000100000
#UCG_String_Amplitude    = $000200000
#UCG_AmplitudeUnit       = $000400000
#UCG_CheckBox_AMEnable   = $000800000
#UCG_AMWaveform          = $001000000
#UCG_ComboBox_AMWaveform = $002000000
#UCG_AMFrequency         = $004000000
#UCG_String_AMFrequency  = $008000000
#UCG_AMMinAmp            = $010000000
#UCG_String_AMMinAmp     = $020000000
#UCG_AMMinAmpUnit        = $040000000
#UCG_AMMaxAmp            = $080000000
#UCG_String_AMMaxAmp     = $100000000
#UCG_AMMaxAmpUnit        = $200000000
#UCG_SettingsAny         = #UCG_Flags | #UCG_Waveform | #UCG_PhaseShift |
                           #UCG_Frequency | #UCG_FMWaveform | #UCG_FMFrequency | #UCG_FMMinFreq | #UCG_FMMaxFreq |
                           #UCG_Amplitude | #UCG_AMWaveform | #UCG_AMFrequency | #UCG_AMMinAmp | #UCG_AMMaxAmp
Procedure UpdateChannelGadgets(*C.Channel)
  
  Shared WaveformGenerator
  
  With *C
    
    ; Eingabe
    
    ;{ Wellenform
    
    ; ComboBox Wellenform
    If \Update & #UCG_ComboBox_Waveform
      Item = GetGadgetState(\ComboBox_Waveform)
      If Item > -1
        \Settings\Waveform = GetGadgetItemData(\ComboBox_Waveform, Item)
        \Update | #UCG_Waveform
      EndIf
    EndIf
    
    ;}
    
    ;{ Stummschaltung
    
    ; CheckBox Stummschaltung
    If \Update & #UCG_CheckBox_Mute
      If GetGadgetState(\CheckBox_Mute) = #PB_Checkbox_Checked
        \Settings\Flags | #WG_Mute
      Else
        \Settings\Flags & ~#WG_Mute
      EndIf
      \Update | #UCG_Flags
    EndIf
    
    ;}
    
    ;{ Phasenverschiebung
    
    ; TrackBar Phasenverschiebung
    If \Update & #UCG_TrackBar_PhaseShift
      \Settings\PhaseShift = (GetGadgetState(\TrackBar_PhaseShift)*360/180-180)*#PI/180
      \Update | #UCG_PhaseShift
    EndIf
    
    ; String Phasenverschiebung
    If \Update & #UCG_String_PhaseShift
      \Settings\PhaseShift = ValDComma(GetGadgetText(\String_PhaseShift)) * #PI / 180
      \Update | #UCG_PhaseShift
    EndIf
    
    ;}
    
    ;{ Frequenz
    
    ; TrackBar Frequenz
    If \Update & #UCG_TrackBar_Frequency
      \Settings\Frequency = 20 * Pow(1000, GetGadgetState(\TrackBar_Frequency)/300)
      \Update | #UCG_Frequency
    EndIf
    
    ; String Frequenz
    If \Update & #UCG_String_Frequency
      \Settings\Frequency = ValDComma(GetGadgetText(\String_Frequency))
      \Update | #UCG_Frequency
    EndIf
    
    ; CheckBox Frequenzmodulation
    If \Update & #UCG_CheckBox_FMEnable
      If GetGadgetState(\CheckBox_FMEnable) = #PB_Checkbox_Checked
        \Settings\Flags | #WG_FMEnable
      Else
        \Settings\Flags & ~#WG_FMEnable
      EndIf
      \Update | #UCG_Flags
    EndIf
    
    ; ComboBox FM-Wellenform
    If \Update & #UCG_ComboBox_FMWaveform
      Item = GetGadgetState(\ComboBox_FMWaveform)
      If Item > -1
        \Settings\FMWaveform = GetGadgetItemData(\ComboBox_FMWaveform, Item)
        \Update | #UCG_FMWaveform
      EndIf
    EndIf
    
    ; String FM-Frequenz
    If \Update & #UCG_String_FMFrequency
      \Settings\FMFrequency = ValDComma(GetGadgetText(\String_FMFrequency))
      \Update | #UCG_FMFrequency
    EndIf
    
    ; String Minimale FM-Frequenz
    If \Update & #UCG_String_FMMinFreq
      \Settings\FMMinFreq = ValDComma(GetGadgetText(\String_FMMinFreq))
      \Update | #UCG_FMMinFreq
    EndIf
    
    ; String Maximale FM-Frequenz
    If \Update & #UCG_String_FMMaxFreq
      \Settings\FMMaxFreq = ValDComma(GetGadgetText(\String_FMMaxFreq))
      \Update | #UCG_FMMaxFreq
    EndIf
    
    ;}
    
    ;{ Amplitude
    
    ; TrackBar Amplitude
    If \Update & #UCG_TrackBar_Amplitude
      Select \AmplitudeUnit
        Case #AmplitudeUnit_Linear
          \Settings\Amplitude = GetGadgetState(\TrackBar_Amplitude) / 300
        Case #AmplitudeUnit_Decibel
          \Settings\Amplitude = Pow(10, -60*(1-GetGadgetState(\TrackBar_Amplitude)/300)/20)
      EndSelect
      \Update | #UCG_Amplitude
    EndIf
    
    ; String Amplitude
    If \Update & #UCG_String_Amplitude
      Select \AmplitudeUnit
        Case #AmplitudeUnit_Linear
          \Settings\Amplitude = ValDComma(GetGadgetText(\String_Amplitude))
        Case #AmplitudeUnit_Decibel
          \Settings\Amplitude = Pow(10, ValDComma(GetGadgetText(\String_Amplitude))/20)
      EndSelect
      \Update | #UCG_Amplitude
    EndIf
    
    ; CheckBox Amplitudenmodulation
    If \Update & #UCG_CheckBox_AMEnable
      If GetGadgetState(\CheckBox_AMEnable) = #PB_Checkbox_Checked
        \Settings\Flags | #WG_AMEnable
      Else
        \Settings\Flags & ~#WG_AMEnable
      EndIf
      \Update | #UCG_Flags
    EndIf
    
    ; ComboBox AM-Wellenform
    If \Update & #UCG_ComboBox_AMWaveform
      Item = GetGadgetState(\ComboBox_AMWaveform)
      If Item > -1
        \Settings\AMWaveform = GetGadgetItemData(\ComboBox_AMWaveform, Item)
        \Update | #UCG_AMWaveform
      EndIf
    EndIf
    
    ; String AM-Frequenz
    If \Update & #UCG_String_AMFrequency
      \Settings\AMFrequency = ValDComma(GetGadgetText(\String_AMFrequency))
      \Update | #UCG_AMFrequency
    EndIf
    
    ; String Minimale AM-Amplitude
    If \Update & #UCG_String_AMMinAmp
      Select \AMMinAmpUnit
        Case #AmplitudeUnit_Linear
          \Settings\AMMinAmp = ValDComma(GetGadgetText(\String_AMMinAmp))
        Case #AmplitudeUnit_Decibel
          \Settings\AMMinAmp = Pow(10, ValDComma(GetGadgetText(\String_AMMinAmp))/20)
      EndSelect
      \Update | #UCG_AMMinAmp
    EndIf
    
    ; String Maximale AM-Amplitude
    If \Update & #UCG_String_AMMaxAmp
      Select \AMMaxAmpUnit
        Case #AmplitudeUnit_Linear
          \Settings\AMMaxAmp = ValDComma(GetGadgetText(\String_AMMaxAmp))
        Case #AmplitudeUnit_Decibel
          \Settings\AMMaxAmp = Pow(10, ValDComma(GetGadgetText(\String_AMMaxAmp))/20)
      EndSelect
      \Update | #UCG_AMMaxAmp
    EndIf
    
    ;}

    ; Verarbeitung
    
    ; Verhindern, dass unzulssige Amplitudeneinheiten eingestellt werden
    ; Einheit wird auf Linear gendert, falls Amplitude auf einen fr dB unzulssigen Wert gendert
    ; wird (beim Laden von Einstellungen) oder Benutzer dB bei unzulssiger Amplitude auswhlt.
    If \Update & (#UCG_Amplitude | #UCG_AmplitudeUnit)
      If \AmplitudeUnit = #AmplitudeUnit_Decibel And \Settings\Amplitude <= 0
        \AmplitudeUnit = #AmplitudeUnit_Linear
        \Update | #UCG_AmplitudeUnit
      EndIf
    EndIf
    If \Update & (#UCG_AMMinAmp | #UCG_AMMinAmpUnit)
      If \AMMinAmpUnit = #AmplitudeUnit_Decibel And \Settings\AMMinAmp <= 0
        \AMMinAmpUnit = #AmplitudeUnit_Linear
        \Update | #UCG_AMMinAmpUnit
      EndIf
    EndIf
    If \Update & (#UCG_AMMaxAmp | #UCG_AMMaxAmpUnit)
      If \AMMaxAmpUnit = #AmplitudeUnit_Decibel And \Settings\AMMaxAmp <= 0
        \AMMaxAmpUnit = #AmplitudeUnit_Linear
        \Update | #UCG_AMMaxAmpUnit
      EndIf
    EndIf
    
    ; Kanaleinstellungen bernehmen
    If \Update & #UCG_SettingsAny
      WaveformGeneratorApplyChannelSettings(@WaveformGenerator, \Index)
    EndIf
    
    ; Ausgabe
    
    ;{ Wellenform
    
    ; ComboBox Wellenform
    If \Update & #UCG_Waveform And Not \Update & #UCG_ComboBox_Waveform
      SetGadgetStateByItemData(\ComboBox_Waveform, \Settings\Waveform)
    EndIf
    
    ;}
    
    ;{ Phasenverschiebung
    
    ; TrackBar Phasenverschiebung
    If \Update & #UCG_PhaseShift And Not \Update & #UCG_TrackBar_PhaseShift
      State = (\Settings\PhaseShift * 180 / #PI + 180) * 180 / 360
      Limit(State, 0, 180)
      SetGadgetState(\TrackBar_PhaseShift, State)
    EndIf
    
    ; String Phasenverschiebung
    If \Update & #UCG_PhaseShift And Not \Update & #UCG_String_PhaseShift
      SetGadgetText(\String_PhaseShift, FS_StrD(\Settings\PhaseShift*180/#PI, 6))
    EndIf
    
    ;}
    
    ;{ Stummschaltung
    
    ; CheckBox Stummschaltung
    If \Update & #UCG_Flags And Not \Update & #UCG_CheckBox_Mute
      If \Settings\Flags & #WG_Mute
        SetGadgetState(\CheckBox_Mute, #PB_Checkbox_Checked)
      Else
        SetGadgetState(\CheckBox_Mute, #PB_Checkbox_Unchecked)
      EndIf
    EndIf
    
    ;}
    
    ;{ Frequenz
    
    ; TrackBar Frequenz
    If \Update & #UCG_Frequency And Not \Update & #UCG_TrackBar_Frequency
      State = 300 * Log(\Settings\Frequency/20) / Log(1000)
      Limit(State, 0, 300)
      SetGadgetState(\TrackBar_Frequency, State)
    EndIf
    
    ; String Frequenz
    If \Update & #UCG_Frequency And Not \Update & #UCG_String_Frequency
      SetGadgetText(\String_Frequency, FS_StrD(\Settings\Frequency, 6))
    EndIf
    
    ; CheckBox Frequenzmodulation
    If \Update & #UCG_Flags And Not \Update & #UCG_CheckBox_FMEnable
      If \Settings\Flags & #WG_FMEnable
        SetGadgetState(\CheckBox_FMEnable, #PB_Checkbox_Checked)
      Else
        SetGadgetState(\CheckBox_FMEnable, #PB_Checkbox_Unchecked)
      EndIf
    EndIf
    
    ; ComboBox FM-Wellenform
    If \Update & #UCG_FMWaveform And Not \Update & #UCG_ComboBox_FMWaveform
      SetGadgetStateByItemData(\ComboBox_FMWaveform, \Settings\FMWaveform)
    EndIf
    
    ; String FM-Frequenz
    If \Update & #UCG_FMFrequency And Not \Update & #UCG_String_FMFrequency
      SetGadgetText(\String_FMFrequency, FS_StrD(\Settings\FMFrequency, 6))
    EndIf
    
    ; String Minimale FM-Frequenz
    If \Update & #UCG_FMMinFreq And Not \Update & #UCG_String_FMMinFreq
      SetGadgetText(\String_FMMinFreq, FS_StrD(\Settings\FMMinFreq, 6))
    EndIf
    
    ; String Maximale FM-Frequenz
    If \Update & #UCG_FMMaxFreq And Not \Update & #UCG_String_FMMaxFreq
      SetGadgetText(\String_FMMaxFreq, FS_StrD(\Settings\FMMaxFreq, 6))
    EndIf
    
    ;}
    
    ;{ Amplitude
    
    ; TrackBar Amplitude
    If \Update & (#UCG_Amplitude | #UCG_AmplitudeUnit) And Not \Update & #UCG_TrackBar_Amplitude
      Select \AmplitudeUnit
        Case #AmplitudeUnit_Linear
          State = \Settings\Amplitude * 300
        Case #AmplitudeUnit_Decibel
          State = 300 * (1 - 20 * Log10(\Settings\Amplitude) / -60)
      EndSelect
      Limit(State, 0, 300)
      SetGadgetState(\TrackBar_Amplitude, State)
    EndIf
    
    ; String Amplitude
    If \Update & (#UCG_Amplitude | #UCG_AmplitudeUnit) And Not \Update & #UCG_String_Amplitude
      Select \AmplitudeUnit
        Case #AmplitudeUnit_Linear
          SetGadgetText(\String_Amplitude, FS_StrD(\Settings\Amplitude, 6))
        Case #AmplitudeUnit_Decibel
          SetGadgetText(\String_Amplitude, FS_StrD(20*Log10(\Settings\Amplitude), 6))
      EndSelect
    EndIf
    
    ; Button Amplituden-Einheit
    If \Update & #UCG_AmplitudeUnit
      Select \AmplitudeUnit
        Case #AmplitudeUnit_Linear
          SetGadgetText(\Button_AmplitudeUnit, "1")
        Case #AmplitudeUnit_Decibel
          SetGadgetText(\Button_AmplitudeUnit, "dB")
      EndSelect
    EndIf
    
    ; CheckBox Amplitudenmodulation
    If \Update & #UCG_Flags And Not \Update & #UCG_CheckBox_AMEnable
      If \Settings\Flags & #WG_AMEnable
        SetGadgetState(\CheckBox_AMEnable, #PB_Checkbox_Checked)
      Else
        SetGadgetState(\CheckBox_AMEnable, #PB_Checkbox_Unchecked)
      EndIf
    EndIf
    
    ; ComboBox AM-Wellenform
    If \Update & #UCG_AMWaveform And Not \Update & #UCG_ComboBox_AMWaveform
      SetGadgetStateByItemData(\ComboBox_AMWaveform, \Settings\AMWaveform)
    EndIf

    ; String AM-Frequenz
    If \Update & #UCG_AMFrequency And Not \Update & #UCG_String_AMFrequency
      SetGadgetText(\String_AMFrequency, FS_StrD(\Settings\AMFrequency, 6))
    EndIf
    
    ; String Minimale AM-Amplitude
    If \Update & (#UCG_AMMinAmp | #UCG_AMMinAmpUnit) And Not \Update & #UCG_String_AMMinAmp
      Select \AMMinAmpUnit
        Case #AmplitudeUnit_Linear
          SetGadgetText(\String_AMMinAmp, FS_StrD(\Settings\AMMinAmp, 6))
        Case #AmplitudeUnit_Decibel
          SetGadgetText(\String_AMMinAmp, FS_StrD(20*Log10(\Settings\AMMinAmp), 6))
      EndSelect
    EndIf
    
    ; Button Minimale AM-Amplituden-Einheit
    If \Update & #UCG_AMMinAmpUnit
      Select \AMMinAmpUnit
        Case #AmplitudeUnit_Linear
          SetGadgetText(\Button_AMMinAmpUnit, "1")
        Case #AmplitudeUnit_Decibel
          SetGadgetText(\Button_AMMinAmpUnit, "dB")
      EndSelect
    EndIf
    
    ; String Maximale AM-Amplitude
    If \Update & (#UCG_AMMaxAmp | #UCG_AMMaxAmpUnit) And Not \Update & #UCG_String_AMMaxAmp
      Select \AMMaxAmpUnit
        Case #AmplitudeUnit_Linear
          SetGadgetText(\String_AMMaxAmp, FS_StrD(\Settings\AMMaxAmp, 6))
        Case #AmplitudeUnit_Decibel
          SetGadgetText(\String_AMMaxAmp, FS_StrD(20*Log10(\Settings\AMMaxAmp), 6))
      EndSelect
    EndIf
    
    ; Button Maximale AM-Amplituden-Einheit
    If \Update & #UCG_AMMaxAmpUnit
      Select \AMMaxAmpUnit
        Case #AmplitudeUnit_Linear
          SetGadgetText(\Button_AMMaxAmpUnit, "1")
        Case #AmplitudeUnit_Decibel
          SetGadgetText(\Button_AMMaxAmpUnit, "dB")
      EndSelect
    EndIf
    
    ;}
    
    \Update = 0
    
  EndWith
  
EndProcedure



; ########################################
;- Programmanfang
; ########################################

; Eigenen Pfad ermitteln
CompilerIf #PB_Editor_CreateExecutable
  SelfPath$ = GetPathPart(ProgramFilename())
CompilerElse
  SelfPath$ = #PB_Compiler_FilePath + "..\"
CompilerEndIf

;- Hauptfenster
If OpenWindow(#Window_Main, #PB_Ignore, #PB_Ignore, 850, 415+GetSystemMetrics_(#SM_CYMENU), "nhWaveGen", #PB_Window_SystemMenu|#PB_Window_MinimizeGadget|#PB_Window_MaximizeGadget|#PB_Window_SizeGadget)
  
  WindowBounds(#Window_Main, 400, 200, #PB_Ignore, #PB_Ignore)
  
  ;- Menleiste
  If CreateMenu(#Menu_MainWnd, WindowID(#Window_Main))
    
    ; Mentitel "Funktionsgenerator"
    MenuTitle("&Funktionsgenerator")
    MenuItem(#MenuItem_MainWndStartStop, "")
    AddKeyboardShortcut(#Window_Main, #PB_Shortcut_Control|#PB_Shortcut_R, #MenuItem_MainWndStartStop)
    MenuBar()
    MenuItem(#MenuItem_MainWndQuit, "&Beenden")
    
    ; Mentitel "Laden"
    MenuTitle("&Laden")
    For n = 0 To #MemoryEntryCount - 1
      MenuItem(#MenuItem_MainWndMemoryLoadFirst+n, "")
      If n < 10
        AddKeyboardShortcut(#Window_Main, #PB_Shortcut_Control|(#PB_Shortcut_0+(n+1)%10), #MenuItem_MainWndMemoryLoadFirst+n)
      EndIf
    Next
    
    ; Mentitel "Speichern"
    MenuTitle("&Speichern")
    For n = 0 To #MemoryEntryCount - 1
      MenuItem(#MenuItem_MainWndMemorySaveFirst+n, "")
      If n < 10
        AddKeyboardShortcut(#Window_Main, #PB_Shortcut_Alt|(#PB_Shortcut_0+(n+1)%10), #MenuItem_MainWndMemorySaveFirst+n)
      EndIf
    Next
    
    ; Mentitel "?"
    MenuTitle("&?")
    MenuItem(#MenuItem_MainWndHelp, "&Hilfe..."+#TAB$+"F1")
    AddKeyboardShortcut(#Window_Main, #PB_Shortcut_F1, #MenuItem_MainWndHelp)
    MenuItem(#MenuItem_MainWndAbout, "&ber "+#AppName+"...")
      
  EndIf

  ;- Popup-Men Amplitudeneinheit
  If CreatePopupMenu(#Menu_AmpUnit)
    MenuItem(#MenuItem_AmpUnitLinear, "1")
    MenuItem(#MenuItem_AmpUnitDecibel, "dB")
  EndIf
  
  ;- Gadgets
  ScrollAreaGadget(#ScrollArea_MainWndChannelGadgets, 0, 0, 100, 100, 50, 50, 50, #PB_ScrollArea_BorderLess)
  CloseGadgetList()
  
  ; Konfiguration laden
  LoadConfig(SelfPath$+#ConfigFileName)

  UpdateMainWindow(#UMW_Size|#UMW_WaveformGeneratorEnable|#UMW_Memory)

  Repeat
    
    Event = WaitWindowEvent()
    
    Select EventWindow()
        
      ;- Ereignisse im Hauptfenster
      Case #Window_Main
    
        Select Event
            
          ;- > Schlieen
          Case #PB_Event_CloseWindow
            Quit = #True
            
          ;- > Grennderung
          Case #PB_Event_SizeWindow
            UpdateMainWindow(#UMW_Size)
            
          ;- > Men-Ereignis
          Case #PB_Event_Menu
            
            Select EventMenu()
                
              ; Funktionsgenerator -> Starten/Anhalten
              Case #MenuItem_MainWndStartStop
                If WaveformGeneratorEnabled(@WaveformGenerator)
                  If WaveformGeneratorStop(@WaveformGenerator)
                    ForEach Channel()
                      FreeChannelGadgets(@Channel())
                      DeleteElement(Channel())
                    Next
                    UpdateMainWindow(#UMW_WaveformGeneratorEnable)
                  Else
                    ErrorMessage("Beim Beenden des Funktionsgenerators ist ein Fehler aufgetreten!")
                  EndIf
                Else
                  If IsWindow(#Window_OutputDevice)
                    SetActiveWindow(#Window_OutputDevice)
                  Else
                    OutputDeviceWindow()
                  EndIf
                EndIf
                
              ; Funktionsgenerator -> Beenden
              Case #MenuItem_MainWndQuit
                Quit = #True
                
              ; Laden
              Case #MenuItem_MainWndMemoryLoadFirst To #MenuItem_MainWndMemoryLoadLast
                Index = EventMenu() - #MenuItem_MainWndMemoryLoadFirst
                ForEach Channel()
                  CopyStructure(@Memory(Index)\Settings[Channel()\Index], Channel()\Settings, WG_ChannelSettings)
                  Channel()\Update | #UCG_SettingsAny
                  UpdateChannelGadgets(@Channel())
                Next
                
              ; Speichern
              Case #MenuItem_MainWndMemorySaveFirst To #MenuItem_MainWndMemorySaveLast
                Index = EventMenu() - #MenuItem_MainWndMemorySaveFirst
                Name$ = Trim(InputRequester("Einstellung speichern", "Bitte geben Sie einen Namen fr die Einstellung ein!", Memory(Index)\Name))
                If Name$ <> ""
                  Memory(Index)\Name = Name$
                  ForEach Channel()
                    CopyStructure(Channel()\Settings, @Memory(Index)\Settings[Channel()\Index], WG_ChannelSettings)
                  Next
                  UpdateMainWindow(#UMW_Memory)
                EndIf

              ; ? -> Hilfe
              Case #MenuItem_MainWndHelp
                FileName$ = SelfPath$ + #HelpFileName
                File = CreateFile(#PB_Any, FileName$)
                If File
                  WriteData(File, ?HelpFile, ?HelpFileEnd-?HelpFile)
                  CloseFile(File)
                  ShellExecute_(#Null, "open", FileName$, #Null, #Null, #SW_SHOWNORMAL)
                Else
                  ErrorMessage("Die Datei "+#DQUOTE$+FileName$+#DQUOTE$+" konnte nicht erstellt werden!")
                EndIf
                
              ; ? -> ber
              Case #MenuItem_MainWndAbout
                If IsWindow(#Window_About)
                  SetActiveWindow(#Window_About)
                Else
                  AboutWindow()
                EndIf
    
              ; Amplitudeneinheit -> 1
              Case #MenuItem_AmpUnitLinear
                With AmpUnitMenuInfo
                  \Unit\i = #AmplitudeUnit_Linear
                  \Channel\Update | \Update
                  UpdateChannelGadgets(\Channel)
                EndWith
                
              ; Amplitudeneinheit -> dB
              Case #MenuItem_AmpUnitDecibel
                With AmpUnitMenuInfo
                  \Unit\i = #AmplitudeUnit_Decibel
                  \Channel\Update | \Update
                  UpdateChannelGadgets(\Channel)
                EndWith
                
            EndSelect
            
          ;- > Gadget-Ereignis
          Case #PB_Event_Gadget
            
            Select EventGadget()
                
              ; ScrollArea Kanal-Gadgets
              Case #ScrollArea_MainWndChannelGadgets
                
              ; Kanal-Gadgets
              Default
                
                ForEach Channel()
                  
                  Select EventGadget()
                    
                    ;{ Wellenform
                      
                    ; ComboBox Wellenform
                    Case Channel()\ComboBox_Waveform
                      Channel()\Update | #UCG_ComboBox_Waveform
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button Wellenform-Datei
                    Case Channel()\Button_WaveformFileSelect
                      FileName$ = OpenFileRequester("", "", "Text- und CSV-Dateien (*.txt;*.csv)|*.txt;*.csv|Alle Dateien (*.*)|*.*", 0)
                      If FileName$ <> ""
                        If WaveformGeneratorReadArbitraryFile(@WaveformGenerator, Channel()\Index, #WG_ArbitraryData_Waveform, FileName$)
                          SetGadgetText(Channel()\String_WaveformFile, GetFilePart(FileName$))
                          Channel()\Settings\Waveform = #WG_Arbitrary
                          Channel()\Update | #UCG_Waveform
                          UpdateChannelGadgets(@Channel())
                        Else
                          ErrorMessage("Beim Einlesen der Datei "+#DQUOTE$+FileName$+#DQUOTE$+" ist ein Fehler aufgetreten!")
                        EndIf
                      EndIf
                      
                    ;}
                      
                    ;{ Stummschaltung
                      
                    ; CheckBox Stummschaltung
                    Case Channel()\CheckBox_Mute
                      Channel()\Update | #UCG_CheckBox_Mute
                      UpdateChannelGadgets(@Channel())
                      
                    ;}
                    
                    ;{ Phasenverschiebung
                    
                    ; TrackBar Phasenverschiebung
                    Case Channel()\TrackBar_PhaseShift
                      Channel()\Update | #UCG_TrackBar_PhaseShift
                      UpdateChannelGadgets(@Channel())
                      
                    ; String Phasenverschiebung
                    Case Channel()\String_PhaseShift
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_PhaseShift
                        UpdateChannelGadgets(@Channel())
                      EndIf
                      
                    ;}
                      
                    ;{ Frequenz
                      
                    ; TrackBar Frequenz
                    Case Channel()\TrackBar_Frequency
                      Channel()\Update | #UCG_TrackBar_Frequency
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button Frequenz +10
                    Case Channel()\Button_FreqUpTen
                      Channel()\Settings\Frequency + 10
                      Channel()\Update | #UCG_Frequency
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button Frequenz +1
                    Case Channel()\Button_FreqUpOne
                      Channel()\Settings\Frequency + 1
                      Channel()\Update | #UCG_Frequency
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button Frequenz +0,1
                    Case Channel()\Button_FreqUpTenth
                      Channel()\Settings\Frequency + 0.1
                      Channel()\Update | #UCG_Frequency
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button Frequenz -0,1
                    Case Channel()\Button_FreqDownTenth
                      Channel()\Settings\Frequency - 0.1
                      Channel()\Update | #UCG_Frequency
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button Frequenz -1
                    Case Channel()\Button_FreqDownOne
                      Channel()\Settings\Frequency - 1
                      Channel()\Update | #UCG_Frequency
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button Frequenz -10
                    Case Channel()\Button_FreqDownTen
                      Channel()\Settings\Frequency - 10
                      Channel()\Update | #UCG_Frequency
                      UpdateChannelGadgets(@Channel())
                      
                    ; String Frequenz
                    Case Channel()\String_Frequency
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_Frequency
                        UpdateChannelGadgets(@Channel())
                      EndIf
                      
                    ; CheckBox Frequenzmodulation
                    Case Channel()\CheckBox_FMEnable
                      Channel()\Update | #UCG_CheckBox_FMEnable
                      UpdateChannelGadgets(@Channel())
                      
                    ; ComboBox FM-Wellenform
                    Case Channel()\ComboBox_FMWaveform
                      Channel()\Update | #UCG_ComboBox_FMWaveform
                      UpdateChannelGadgets(@Channel())
                      
;                     ; Button FM-Datei
;                     Case Channel()\Button_FMFileSelect
;                       FileName$ = OpenFileRequester("", "", "Text- und CSV-Dateien (*.txt;*.csv)|*.txt;*.csv|Alle Dateien (*.*)|*.*", 0)
;                       If FileName$ <> ""
;                         If WaveformGeneratorReadArbitraryFile(@WaveformGenerator, @Channel()\Channel\ArbitraryFMWaveform, FileName$)
;                           SetGadgetText(Channel()\String_FMFile, GetFilePart(FileName$)+" ("+Str(Channel()\Channel\ArbitraryFMWaveform\PointCount)+" Punkte)")
;                           Channel()\Settings\FMWaveform = #WG_Arbitrary
;                           Channel()\Update | #UCG_FMWaveform
;                           UpdateChannelGadgets(@Channel())
;                         Else
;                           ErrorMessage("Beim Einlesen der Datei "+#DQUOTE$+FileName$+#DQUOTE$+" ist ein Fehler aufgetreten!")
;                         EndIf
;                       EndIf
                      
                    ; String FM-Frequenz
                    Case Channel()\String_FMFrequency
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_FMFrequency
                        UpdateChannelGadgets(@Channel())
                      EndIf
                      
                    ; String Minimale FM-Frequenz
                    Case Channel()\String_FMMinFreq
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_FMMinFreq
                        UpdateChannelGadgets(@Channel())
                      EndIf
    
                    ; String Maximale FM-Frequenz
                    Case Channel()\String_FMMaxFreq
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_FMMaxFreq
                        UpdateChannelGadgets(@Channel())
                      EndIf
                      
                    ;}
                    
                    ;{ Amplitude
                    
                    ; TrackBar Amplitude
                    Case Channel()\TrackBar_Amplitude
                      Channel()\Update | #UCG_TrackBar_Amplitude
                      UpdateChannelGadgets(@Channel())
                      
                    ; String Amplitude
                    Case Channel()\String_Amplitude
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_Amplitude
                        UpdateChannelGadgets(@Channel())
                      EndIf
    
                    ; Button Amplitudeneinheit
                    Case Channel()\Button_AmplitudeUnit
                      With AmpUnitMenuInfo
                        \Unit = @Channel()\AmplitudeUnit
                        \Channel = @Channel()
                        \Update = #UCG_AmplitudeUnit
                      EndWith
                      DisplayPopupMenuBelowGadget(#Menu_AmpUnit, WindowID(#Window_Main), Channel()\Button_AmplitudeUnit)
                      
                    ; CheckBox Amplitudenmodulation
                    Case Channel()\CheckBox_AMEnable
                      Channel()\Update | #UCG_CheckBox_AMEnable
                      UpdateChannelGadgets(@Channel())
                      
                    ; ComboBox AM-Wellenform
                    Case Channel()\ComboBox_AMWaveform
                      Channel()\Update | #UCG_ComboBox_AMWaveform
                      UpdateChannelGadgets(@Channel())
                      
                    ; Button AM-Datei
                    Case Channel()\Button_AMFileSelect
                      FileName$ = OpenFileRequester("", "", "Text- und CSV-Dateien (*.txt;*.csv)|*.txt;*.csv|Alle Dateien (*.*)|*.*", 0)
                      If FileName$ <> ""
                        If WaveformGeneratorReadArbitraryFile(@WaveformGenerator, Channel()\Index, #WG_ArbitraryData_AMWaveform, FileName$)
                          SetGadgetText(Channel()\String_AMFile, GetFilePart(FileName$))
                          Channel()\Settings\AMWaveform = #WG_Arbitrary
                          Channel()\Update | #UCG_AMWaveform
                          UpdateChannelGadgets(@Channel())
                        Else
                          ErrorMessage("Beim Einlesen der Datei "+#DQUOTE$+FileName$+#DQUOTE$+" ist ein Fehler aufgetreten!")
                        EndIf
                      EndIf
                      
                    ; String AM-Frequenz
                    Case Channel()\String_AMFrequency
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_AMFrequency
                        UpdateChannelGadgets(@Channel())
                      EndIf
                      
                    ; String Minimale AM-Amplitude
                    Case Channel()\String_AMMinAmp
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_AMMinAmp
                        UpdateChannelGadgets(@Channel())
                      EndIf
    
                    ; Button Minimale AM-Amplituden-Einheit
                    Case Channel()\Button_AMMinAmpUnit
                      With AmpUnitMenuInfo
                        \Unit = @Channel()\AMMinAmpUnit
                        \Channel = @Channel()
                        \Update = #UCG_AMMinAmpUnit
                      EndWith
                      DisplayPopupMenuBelowGadget(#Menu_AmpUnit, WindowID(#Window_Main), Channel()\Button_AMMinAmpUnit)
                      
                    ; String Maximale AM-Amplitude
                    Case Channel()\String_AMMaxAmp
                      If EventType() = #PB_EventType_Change
                        Channel()\Update | #UCG_String_AMMaxAmp
                        UpdateChannelGadgets(@Channel())
                      EndIf
    
                    ; Button Maximale AM-Amplituden-Einheit
                    Case Channel()\Button_AMMaxAmpUnit
                      With AmpUnitMenuInfo
                        \Unit = @Channel()\AMMaxAmpUnit
                        \Channel = @Channel()
                        \Update = #UCG_AMMaxAmpUnit
                      EndWith
                      DisplayPopupMenuBelowGadget(#Menu_AmpUnit, WindowID(#Window_Main), Channel()\Button_AMMaxAmpUnit)
                      
                    ;}
                    
                    Default
                      Continue
                    
                  EndSelect
                  
                  Break ; Schleife verlassen, sobald Gadget gefunden wurde
                  
                Next
                
            EndSelect
            
        EndSelect
        
    ;- Ereignisse im Ausgabegertefenster
    Case #Window_OutputDevice
    
      Select Event
          
        ;- > Gadget-Ereignis
        Case #PB_Event_Gadget
        
          Select EventGadget()
              
            ; Button Abbrechen
            Case #Button_OutDevWndCancel
              CloseWindow(#Window_OutputDevice)
              
            ; Button OK
            Case #Button_OutDevWndOK
              If GetOutputDevice()
                For n = 0 To WaveformGenerator\ChannelCount - 1
                  With WaveformGenerator\Channel[n]\Settings
                    \Flags = #WG_Mute
                    \Waveform = #WG_Sine
                    \PhaseShift = 0
                    \Frequency = 440
                    \FMWaveform = #WG_Sine
                    \FMFrequency = 1
                    \FMMinFreq = 440
                    \FMMaxFreq = 880
                    \Amplitude = 0.1
                    \AMWaveform = #WG_Sine
                    \AMFrequency = 1
                    \AMMinAmp = 0.001
                    \AMMaxAmp = 0.1
                  EndWith
                Next
                WaveformGeneratorApplyChannelSettings(@WaveformGenerator)
                If WaveformGeneratorStart(@WaveformGenerator)
                  CloseWindow(#Window_OutputDevice)
                  UpdateMainWindow(#UMW_WaveformGeneratorEnable)
                  OpenGadgetList(#ScrollArea_MainWndChannelGadgets)
                  For n = 0 To WaveformGenerator\ChannelCount - 1
                    If AddElement(Channel())
                      Channel()\Index = n
                      Channel()\AmplitudeUnit = #AmplitudeUnit_Decibel
                      Channel()\AMMinAmpUnit = #AmplitudeUnit_Decibel
                      Channel()\AMMaxAmpUnit = #AmplitudeUnit_Decibel
                      Channel()\Channel = @WaveformGenerator\Channel[n]
                      Channel()\Settings = @WaveformGenerator\Channel[n]\Settings
                      CreateChannelGadgets(@Channel())
                      Channel()\Update | (#UCG_AmplitudeUnit | #UCG_AMMinAmpUnit | #UCG_AMMaxAmpUnit | #UCG_SettingsAny)
                      UpdateChannelGadgets(@Channel())
                    EndIf
                  Next
                  CloseGadgetList()
                Else
                  ErrorMessage("Beim Starten des Funktionsgenerators ist ein Fehler aufgetreten!")
                EndIf
              Else
                ErrorMessage("Unzulssige Einstellungen! Mglicherweise ist die Anzahl der ausgewhlten Kanle unzulssig.")
              EndIf

          EndSelect
          
      EndSelect
        
    ;- Ereignisse im Infofenster
    Case #Window_About
      
      Select Event
          
        ;- > Gadget-Ereignis
        Case #PB_Event_Gadget
          
          Select EventGadget()
              
            ; Hyperlink
            Case #HyperLink_AboutWndLink
              ShellExecute_(#Null, "open", #AppHomepage, #Null, #Null, #SW_SHOWNORMAL)
              
            ; Button Schlieen
            Case #Button_AboutWndClose
              CloseWindow(#Window_About)
              
          EndSelect
          
      EndSelect

  EndSelect
    
  Until Quit
  
  ; WaveformGenerator anhalten
  WaveformGeneratorStop(@WaveformGenerator)
  
  ; Konfiguration speichern
  SaveConfig(SelfPath$+#ConfigFileName)
  
EndIf

End



; ########################################
;- Binrdaten
; ########################################

DataSection
  ; Kanalnamen
  ChannelNames:
  Data.s "Front Left", "Front Right", "Front Center", "Low Frequency"
  Data.s "Back Left", "Back Right", "Front Left of Center", "Front Right of Center"
  Data.s "Back Center", "Side Left", "Side Right", "Top Center"
  Data.s "Top Front Left", "Top Front Center", "Top Front Right", "Top Back Left"
  Data.s "Top Back Center", "Top Back Right"
  ; Hilfedatei
  HelpFile:
  IncludeBinary "help\help.htm"
  HelpFileEnd:
EndDataSection



; IDE Options = PureBasic 5.31 (Windows - x86)
; Folding = AAAAAAw
; EnableUnicode
; EnableThread
; EnableXP
; UseIcon = nhwavegen.ico
; Executable = ..\nhwavegen.exe
; CurrentDirectory = ..\
; DisablePurifier = 4,4,64,4
; EnableBuildCount = 116
; EnableExeConstant
; IncludeVersionInfo
; VersionField0 = 0,8,0,%BUILDCOUNT
; VersionField1 = 0,8,0,%BUILDCOUNT
; VersionField2 = Nils Hintze
; VersionField3 = nhWaveGen
; VersionField4 = 0.8.0-%BUILDCOUNT (%dd.%mm.%yyyy)
; VersionField5 = 0.8.0-%BUILDCOUNT (%dd.%mm.%yyyy)
; VersionField6 = nhWaveGen
; VersionField7 = nhwavegen
; VersionField8 = nhwavegen.exe
; VersionField9 = © Nils Hintze
; VersionField13 = webmaster@elektronikpage.net
; VersionField14 = http://www.elektronikpage.net/nhwavegen/
; VersionField15 = VOS_NT_WINDOWS32
; VersionField16 = VFT_APP
; VersionField17 = 0407 German (Standard)